/**
 *
 */
package de.intevation.gnv.geobackend.base.connectionpool;

import java.util.Properties;

import org.apache.log4j.Logger;

import de.intevation.gnv.geobackend.sde.connectionpool.ArcSDEConnectionPool;

/**
 * Factoryimplementation for the Interface ConnectionPool.
 * This factory delivers Instances of the Interface ConnectionPool.
 * @author Tim Englich <tim.englich@intevation.de>
 */
public class ConnectionPoolFactory {

    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger.getLogger(ConnectionPoolFactory.class);


    /**
     * The singleton Instance of this Factory.
     */
    private static ConnectionPoolFactory instance = null;

    /**
     * The ConnectionPool providing the Connections to the DatabaseBackends
     */
    private ConnectionPool connectionPool = null;

    /**
     * Basic-Constructor of this Class
     */
    private ConnectionPoolFactory() {
    	super();
    }

    /**
     * This Method provides an singleton Instance of this Class.
     * @return an singleton Instance of this Class
     */
    public static ConnectionPoolFactory getInstance(){
        if (instance == null){
        	instance = new ConnectionPoolFactory();
        }
        return instance;
    }

    /**
     * Getting the ConnectionPool
     * @return the ConnectionPool
     */
    public ConnectionPool getConnectionPool(){
        return this.connectionPool;
    }

    /**
     * Initializes the ConnectionPool.
     * Should only be called once on system startup
     * @param properties the Properties for the Individual Configuration of the ConnectionPool
     */
    public void initializeConnectionPool(Properties properties){
        log.debug("ConnectionPoolFactory.initializeConnectionPool");
        if (this.connectionPool == null){
            //TODO: Here it might be possible to switch the ConnectionPoolInstance using a FLAG.
            this.connectionPool = new ArcSDEConnectionPool();
            this.connectionPool.initialize(properties);
        }else{
            log.warn("ConnectionPool already initialized");
        }
    }

    /**
     * Checks if the ConnectionPool has already been initialized.
     * @return true if the ConnectionPool is initialized.
     */
    public boolean isInitialized(){
        return this.connectionPool != null;
    }
}
